@@ -8,10 +8,13 @@
 #include "settingsmanager.h"
 
 #include <QCoroFuture>
+#include <QDBusConnection>
+#include <QDBusReply>
 #include <QLocale>
 #include <QTimer>
 #include <QtConcurrent>
 
+#include <KIO/CommandLauncherJob>
 #include <KLocalizedString>
 #include <KNotification>
 #include <KPeople/PersonData>
@@ -20,6 +23,7 @@
 #include <global.h>
 
 #include <QCoroDBusPendingReply>
+#include <QCoroFuture>
 
 static bool isScreenSaverActive()
 {
@@ -42,12 +46,8 @@ ChannelLogger::ChannelLogger(std::optional<QString> &modemPath, QObject *parent)
 
     PhoneNumber::setCountryCode(countryCode());
 
-    m_ownNumber = PhoneNumber(ownNumber());
-
     m_database.migrate();
 
-    checkMessages();
-
     connect(&ModemController::instance(), &ModemController::messageAdded, this, [this](ModemManager::Sms::Ptr msg) {
         handleIncomingMessage(msg);
     });
@@ -72,6 +72,9 @@ ChannelLogger::ChannelLogger(std::optional<QString> &modemPath, QObject *parent)
 
 void ChannelLogger::checkMessages()
 {
+    // update own number
+    m_ownNumber = PhoneNumber(ownNumber());
+
     // for any unhandled messages
     const ModemManager::Sms::List messages = ModemController::instance().messages();
     for (const ModemManager::Sms::Ptr &msg : messages) {
@@ -81,11 +84,13 @@ void ChannelLogger::checkMessages()
     }
 }
 
-QString ChannelLogger::ownNumber() {
+QString ChannelLogger::ownNumber()
+{
     return ModemController::instance().ownNumber();
 }
 
-QString ChannelLogger::countryCode() {
+QString ChannelLogger::countryCode()
+{
     QString countryCode = ModemController::instance().countryCode;
 
     if (countryCode.isEmpty()) {
@@ -107,8 +112,8 @@ void ChannelLogger::handleIncomingMessage(ModemManager::Sms::Ptr msg)
 
     const PhoneNumberList phoneNumberList = PhoneNumberList(number);
 
-    //TODO check if blocked number
-    // use phonebook for blocked number storage so can be shared with dialer?
+    // TODO check if blocked number
+    //  use phonebook for blocked number storage so can be shared with dialer?
 
     // text and data are not valid at the same time
     if (!text.isEmpty()) {
@@ -151,11 +156,11 @@ void ChannelLogger::handleIncomingMessage(ModemManager::Sms::Ptr msg)
             if (!mmsMessage.messageId.isEmpty()) {
                 m_database.updateMessageDeliveryReport(mmsMessage.messageId);
             }
-        } else if (mmsMessage.messageType == SL("m-read-orig-ind"))  {
+        } else if (mmsMessage.messageType == SL("m-read-orig-ind")) {
             if (!mmsMessage.messageId.isEmpty()) {
                 m_database.updateMessageReadReport(mmsMessage.messageId, PhoneNumber(mmsMessage.from));
             }
-        } else if (mmsMessage.messageType == SL("m-cancel-req"))  {
+        } else if (mmsMessage.messageType == SL("m-cancel-req")) {
             sendCancelResponse(mmsMessage.transactionId);
         } else {
             qDebug() << "Unknown message type:" << mmsMessage.messageType;
@@ -168,19 +173,17 @@ void ChannelLogger::handleIncomingMessage(ModemManager::Sms::Ptr msg)
 
 void ChannelLogger::createDownloadNotification(const MmsMessage &mmsMessage)
 {
-    saveMessage(
-        PhoneNumberList(mmsMessage.from),
-        mmsMessage.date,
-        QString(), // text
-        QString(), // attachments
-        QString(), // smil
-        QString(), // fromNumber
-        QString(), // messageId
-        true, // pendingDownload
-        mmsMessage.contentLocation,
-        mmsMessage.expiry,
-        mmsMessage.messageSize
-    );
+    saveMessage(PhoneNumberList(mmsMessage.from),
+                mmsMessage.date,
+                QString(), // text
+                QString(), // attachments
+                QString(), // smil
+                QString(), // fromNumber
+                QString(), // messageId
+                true, // pendingDownload
+                mmsMessage.contentLocation,
+                mmsMessage.expiry,
+                mmsMessage.messageSize);
 
     // this is important, otherwise an MMSC server may send repeated notifications
     if (m_dataConnected) {
@@ -214,25 +217,23 @@ void ChannelLogger::handleDownloadedMessage(const QByteArray &response, const QS
     // fromNumber is only useful to know in group conversations
     const QString fromNumber = mmsMessage.to.length() > 1 ? mmsMessage.from : QString();
 
-    saveMessage(
-        mmsMessage.phoneNumberList,
-        mmsMessage.date,
-        mmsMessage.text,
-        mmsMessage.attachments,
-        mmsMessage.smil,
-        fromNumber,
-        mmsMessage.messageId,
-        false,
-        url,
-        expires,
-        response.size()
-    );
+    saveMessage(mmsMessage.phoneNumberList,
+                mmsMessage.date,
+                mmsMessage.text,
+                mmsMessage.attachments,
+                mmsMessage.smil,
+                fromNumber,
+                mmsMessage.messageId,
+                false,
+                url,
+                expires,
+                response.size());
 }
 
-void ChannelLogger::addMessage(const Message &message)
+QCoro::Task<void> ChannelLogger::addMessage(const Message &message)
 {
     // save to database
-    m_database.addMessage(message);
+    co_await m_database.addMessage(message);
 
     // add message to open conversation
     if (!message.sentByMe) {
@@ -246,25 +247,23 @@ void ChannelLogger::updateMessage(const Message &message)
     Q_EMIT messageUpdated(message.phoneNumberList.toString(), message.id);
 }
 
-void ChannelLogger::saveMessage(
-    const PhoneNumberList &phoneNumberList,
-    const QDateTime &datetime,
-    const QString &text,
-    const QString &attachments,
-    const QString &smil,
-    const QString &fromNumber,
-    const QString &messageId,
-    const bool pendingDownload,
-    const QString &contentLocation,
-    const QDateTime &expires,
-    const int size
-)
+QCoro::Task<void> ChannelLogger::saveMessage(const PhoneNumberList &phoneNumberList,
+                                             const QDateTime &datetime,
+                                             const QString &text,
+                                             const QString &attachments,
+                                             const QString &smil,
+                                             const QString &fromNumber,
+                                             const QString &messageId,
+                                             const bool pendingDownload,
+                                             const QString &contentLocation,
+                                             const QDateTime &expires,
+                                             const int size)
 {
     Message message;
     message.text = text;
     message.sentByMe = false; // SMS doesn't have any kind of synchronization, so received messages are always from the chat partner.
     message.datetime = datetime;
-    message.deliveryStatus =  MessageState::Received; // It arrived, soo
+    message.deliveryStatus = MessageState::Received; // It arrived, soo
     message.phoneNumberList = phoneNumberList;
     message.id = Database::generateRandomId();
     message.read = message.phoneNumberList == m_disabledNotificationNumber;
@@ -289,31 +288,31 @@ void ChannelLogger::saveMessage(
         message.datetime = QDateTime::currentDateTime();
     }
 
-    if (handleTapbackReaction(message, message.fromNumber.isEmpty() ? message.phoneNumberList.toString() : message.fromNumber)) {
+    if (co_await handleTapbackReaction(message, message.fromNumber.isEmpty() ? message.phoneNumberList.toString() : message.fromNumber)) {
         updateMessage(message);
 
         if (SettingsManager::self()->ignoreTapbacks()) {
-            return;
+            co_return;
         }
     } else {
-        addMessage(message);
+        co_await addMessage(message);
     }
 
-    //TODO add setting to turn off notifications for multiple chats in addition to current chat
+    // TODO add setting to turn off notifications for multiple chats in addition to current chat
     if (message.phoneNumberList == m_disabledNotificationNumber) {
         if (!isScreenSaverActive()) {
-            return;
+            co_return;
         }
     }
 
     createNotification(message);
 }
 
-void ChannelLogger::sendMessage(const QString &numbers, const QString & id, const QString &text, const QStringList &files, const qint64 &totalSize)
+void ChannelLogger::sendMessage(const QString &numbers, const QString &id, const QString &text, const QStringList &files, const qint64 &totalSize)
 {
     PhoneNumberList phoneNumberList = PhoneNumberList(numbers);
 
-    [this, phoneNumberList, id, text, files, totalSize] () -> QCoro::Task<void> {
+    [this, phoneNumberList, id, text, files, totalSize]() -> QCoro::Task<void> {
         QString result;
         // check if it is a MMS message
         if (phoneNumberList.size() > 1 || files.length() > 0) {
@@ -349,6 +348,11 @@ void ChannelLogger::sendMessage(const QString &numbers, const QString & id, cons
     }();
 }
 
+void ChannelLogger::sendTapback(const QString &numbers, const QString &id, const QString &tapback, const bool &isRemoved)
+{
+    sendTapbackHandler(numbers, id, tapback, isRemoved);
+}
+
 QCoro::Task<QString> ChannelLogger::sendMessageSMS(const PhoneNumber &phoneNumber, const QString &id, const QString &text)
 {
     ModemManager::ModemMessaging::Message m;
@@ -365,7 +369,7 @@ QCoro::Task<QString> ChannelLogger::sendMessageSMS(const PhoneNumber &phoneNumbe
     message.deliveryStatus = MessageState::Pending;
 
     // add message to database
-    addMessage(message);
+    co_await addMessage(message);
 
     auto maybeReply = ModemController::instance().createMessage(m);
 
@@ -389,36 +393,36 @@ QCoro::Task<QString> ChannelLogger::sendMessageSMS(const PhoneNumber &phoneNumbe
         qDebug() << "state changed" << mmMessage->state();
 
         switch (mmMessage->state()) {
-            case MM_SMS_STATE_SENT:
-                // The message was successfully sent
-                m_database.updateMessageDeliveryState(message.id, MessageState::Sent);
-                updateMessage(message);
-                break;
-            case MM_SMS_STATE_RECEIVED:
-                // The message has been completely received
-                // Should not happen
-                qWarning() << "Received a message we sent";
-                break;
-            case MM_SMS_STATE_RECEIVING:
-                // The message is being received but is not yet complete
-                // Should not happen
-                qWarning() << "Receiving a message we sent";
-                break;
-            case MM_SMS_STATE_SENDING:
-                // The message is queued for delivery
-                m_database.updateMessageDeliveryState(message.id, MessageState::Pending);
-                updateMessage(message);
-                break;
-            case MM_SMS_STATE_STORED:
-                // The message has been neither received nor yet sent
-                m_database.updateMessageDeliveryState(message.id, MessageState::Pending);
-                updateMessage(message);
-                break;
-            case MM_SMS_STATE_UNKNOWN:
-                // State unknown or not reportable
-                m_database.updateMessageDeliveryState(message.id, MessageState::Unknown);
-                updateMessage(message);
-                break;
+        case MM_SMS_STATE_SENT:
+            // The message was successfully sent
+            m_database.updateMessageDeliveryState(message.id, MessageState::Sent);
+            updateMessage(message);
+            break;
+        case MM_SMS_STATE_RECEIVED:
+            // The message has been completely received
+            // Should not happen
+            qWarning() << "Received a message we sent";
+            break;
+        case MM_SMS_STATE_RECEIVING:
+            // The message is being received but is not yet complete
+            // Should not happen
+            qWarning() << "Receiving a message we sent";
+            break;
+        case MM_SMS_STATE_SENDING:
+            // The message is queued for delivery
+            m_database.updateMessageDeliveryState(message.id, MessageState::Pending);
+            updateMessage(message);
+            break;
+        case MM_SMS_STATE_STORED:
+            // The message has been neither received nor yet sent
+            m_database.updateMessageDeliveryState(message.id, MessageState::Pending);
+            updateMessage(message);
+            break;
+        case MM_SMS_STATE_UNKNOWN:
+            // State unknown or not reportable
+            m_database.updateMessageDeliveryState(message.id, MessageState::Unknown);
+            updateMessage(message);
+            break;
         }
     });
 
@@ -441,7 +445,8 @@ QCoro::Task<QString> ChannelLogger::sendMessageSMS(const PhoneNumber &phoneNumbe
     co_return QString();
 }
 
-QCoro::Task<QString> ChannelLogger::sendMessageMMS(const PhoneNumberList &phoneNumberList, const QString &id, const QString &text, const QStringList &files, const qint64 totalSize)
+QCoro::Task<QString>
+ChannelLogger::sendMessageMMS(const PhoneNumberList &phoneNumberList, const QString &id, const QString &text, const QStringList &files, const qint64 totalSize)
 {
     Message message;
     message.phoneNumberList = phoneNumberList;
@@ -467,9 +472,9 @@ QCoro::Task<QString> ChannelLogger::sendMessageMMS(const PhoneNumberList &phoneN
     updateMessage(message);
 
     // add message to database
-    addMessage(message);
+    co_await addMessage(message);
 
-    //send message
+    // send message
     const QByteArray response = co_await uploadMessage(data);
     if (response.isEmpty()) {
         m_database.updateMessageDeliveryState(message.id, MessageState::Failed);
@@ -522,6 +527,7 @@ void ChannelLogger::createNotification(Message &message)
 {
     auto *notification = new KNotification(QStringLiteral("incomingMessage"));
     notification->setComponentName(SL("spacebar"));
+    notification->setIconName(SL("message-new"));
 
     QString title = i18n("New message");
     if (SettingsManager::self()->showSenderInfo()) {
@@ -548,23 +554,30 @@ void ChannelLogger::createNotification(Message &message)
                 QList<QUrl> urls;
                 for (const auto &item : items) {
                     const QString local = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation);
-                    const QString folder = QString::number(qHash(message.phoneNumberList.toString()));
+                    const QString folder = QString::number(hash(message.phoneNumberList.toString()));
                     const QString fileName = item.toObject()[SL("fileName")].toString();
                     urls.append(QUrl::fromLocalFile(local + SL("/spacebar/attachments/") + folder + SL("/") + fileName));
                 }
                 notification->setUrls(urls);
             }
+        } else {
+            notification->setText(notificationText);
         }
     }
 
-    notification->setDefaultAction(i18nc("@action open message in application", "Open"));
-    notification->sendEvent();
-
     // copy current pointer to notification, otherwise this would just close the most recent one.
-    connect(notification, &KNotification::defaultActivated, this, [notification]() {
+    auto openApp = [notification, message]() {
         notification->close();
-        QProcess::startDetached(SL("spacebar"), QStringList{});
-    });
+        auto *job = new KIO::CommandLauncherJob(SL("spacebar"), {message.phoneNumberList.toString()});
+        job->setStartupId(notification->xdgActivationToken().toUtf8());
+        job->setDesktopName(SL("org.kde.spacebar"));
+        job->start();
+    };
+
+    auto defaultAction = notification->addDefaultAction(i18nc("@action open message in application", "Open"));
+    connect(defaultAction, &KNotificationAction::activated, this, openApp);
+
+    notification->sendEvent();
 }
 
 void ChannelLogger::disableNotificationsForNumber(const QString &numbers)
@@ -572,34 +585,40 @@ void ChannelLogger::disableNotificationsForNumber(const QString &numbers)
     m_disabledNotificationNumber = PhoneNumberList(numbers);
 }
 
-bool ChannelLogger::handleTapbackReaction(Message &message, const QString &reactNumber)
+QCoro::Task<bool> ChannelLogger::handleTapbackReaction(Message &message, const QString &reactNumber)
 {
     for (const auto &tapback : TAPBACK_REMOVED) {
         if (message.text.startsWith(tapback)) {
-            return saveTapback(message, reactNumber, tapback, TAPBACK_REMOVED, false, false);
+            co_return co_await saveTapback(message, reactNumber, tapback, TAPBACK_REMOVED, false, false);
         } else if (message.text == tapback.left(tapback.length() - 1) + SL("an image")) {
-            return saveTapback(message, reactNumber, tapback, TAPBACK_REMOVED, false, true);
+            co_return co_await saveTapback(message, reactNumber, tapback, TAPBACK_REMOVED, false, true);
         }
     }
 
     for (const auto &tapback : TAPBACK_ADDED) {
         if (message.text.startsWith(tapback)) {
-            return saveTapback(message, reactNumber, tapback, TAPBACK_ADDED, true, false);
+            co_return co_await saveTapback(message, reactNumber, tapback, TAPBACK_ADDED, true, false);
         } else if (message.text == tapback.left(tapback.length() - 1) + SL("an image")) {
-            return saveTapback(message, reactNumber, tapback, TAPBACK_ADDED, true, true);
+            co_return co_await saveTapback(message, reactNumber, tapback, TAPBACK_ADDED, true, true);
         }
     }
 
-    return false;
+    co_return false;
 }
 
-bool ChannelLogger::saveTapback(Message &message, const QString &reactNumber, const QStringView &tapback, std::span<const QStringView> list, const bool &isAdd, const bool &isImage)
+QCoro::Task<bool> ChannelLogger::saveTapback(Message &message,
+                                             const QString &reactNumber,
+                                             const QStringView &tapback,
+                                             std::span<const QStringView> list,
+                                             const bool &isAdd,
+                                             const bool &isImage)
 {
     const QString searchText = isImage ? SL("") : message.text.mid(tapback.length(), message.text.length() - tapback.length() - 1);
-    const QString id = isImage ? m_database.lastMessageWithAttachment(message.phoneNumberList) : m_database.lastMessageWithText(message.phoneNumberList, searchText);
+    const auto id = isImage ? co_await m_database.lastMessageWithAttachment(message.phoneNumberList)
+                            : co_await m_database.lastMessageWithText(message.phoneNumberList, searchText);
 
-    if (!id.isEmpty()) {
-        Message msg = m_database.messagesForNumber(message.phoneNumberList, id).first();
+    if (id) {
+        Message msg = (co_await m_database.messagesForNumber(message.phoneNumberList, *id)).front();
         QJsonObject reactions = QJsonDocument::fromJson(msg.tapbacks.toUtf8()).object();
         QJsonArray numbers;
         const QJsonValue number = QJsonValue(reactNumber);
@@ -613,7 +632,7 @@ bool ChannelLogger::saveTapback(Message &message, const QString &reactNumber, co
                     if (numbers.at(i) == number) {
                         numbers.removeAt(i);
 
-                        if(numbers.isEmpty()) {
+                        if (numbers.isEmpty()) {
                             reactions.remove(keyToRemove);
                         } else {
                             reactions[keyToRemove] = numbers;
@@ -635,7 +654,6 @@ bool ChannelLogger::saveTapback(Message &message, const QString &reactNumber, co
             reactions.insert(TAPBACK_KEYS[idx], numbers);
         }
 
-
         if (reactions.isEmpty()) {
             msg.tapbacks = QString();
         } else {
@@ -644,18 +662,18 @@ bool ChannelLogger::saveTapback(Message &message, const QString &reactNumber, co
             msg.tapbacks = QString::fromUtf8(jsonDoc.toJson(QJsonDocument::Compact));
         }
 
-        m_database.updateMessageTapbacks(id, msg.tapbacks);
+        m_database.updateMessageTapbacks(*id, msg.tapbacks);
 
         message.id = msg.id;
-        return true;
+        co_return true;
     }
 
-    return false;
+    co_return false;
 }
 
-void ChannelLogger::sendTapback(const QString &numbers, const QString &id, const QString &tapback, const bool &isRemoved)
+QCoro::Task<void> ChannelLogger::sendTapbackHandler(const QString &numbers, const QString &id, const QString &tapback, const bool &isRemoved)
 {
-    Message message = m_database.messagesForNumber(PhoneNumberList(numbers), id).first();
+    Message message = (co_await m_database.messagesForNumber(PhoneNumberList(numbers), id)).front();
     const int idx = std::find(TAPBACK_KEYS.cbegin(), TAPBACK_KEYS.cend(), tapback) - TAPBACK_KEYS.cbegin();
 
     if (message.attachments.isEmpty()) {
@@ -684,13 +702,13 @@ void ChannelLogger::sendTapback(const QString &numbers, const QString &id, const
 
         if (!maybeReply) {
             qDebug() << "No modem";
-            return;
+            co_return;
         }
 
         const QDBusReply<QDBusObjectPath> msgPathResult = *maybeReply;
 
         if (!msgPathResult.isValid()) {
-            return;
+            co_return;
         }
 
         ModemManager::Sms::Ptr mmMessage = QSharedPointer<ModemManager::Sms>::create(msgPathResult.value().path());
@@ -699,7 +717,7 @@ void ChannelLogger::sendTapback(const QString &numbers, const QString &id, const
 
         if (!sendResult.isValid()) {
             qDebug() << sendResult.error().message();
-            return;
+            co_return;
         }
     }
 }
@@ -717,11 +735,11 @@ QCoro::Task<QByteArray> ChannelLogger::uploadMessage(const QByteArray &data)
         co_return BL("");
     }
 
-    #if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
     const QByteArray response = co_await QtConcurrent::run(&ECurl::networkRequest, &m_curl, url, data);
-    #else
+#else
     const QByteArray response = co_await QtConcurrent::run(&m_curl, &ECurl::networkRequest, url, data);
-    #endif
+#endif
 
     if (response.isNull()) {
         co_return QByteArray();
@@ -734,11 +752,11 @@ QCoro::Task<void> ChannelLogger::downloadMessage(const MmsMessage message)
 {
     const QString url = message.contentLocation;
 
-    #if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
     const QByteArray response = co_await QtConcurrent::run(&ECurl::networkRequest, &m_curl, url, BL(""));
-    #else
+#else
     const QByteArray response = co_await QtConcurrent::run(&m_curl, &ECurl::networkRequest, url, BL(""));
-    #endif
+#endif
 
     if (response.isNull()) {
         if (!message.databaseId.isEmpty()) {
